@@ -16,7 +16,7 @@ utility.
 Note: The script must be run under Windows if any file should be signed.
 
 Usage:
-  deploy [[-e] -p <passfile>] <specfile> <package_name> <destination>
+  deploy [[-e] -p <passfile>] [--dry-run] <specfile> <package_name> <destination>
   deploy -h | --help
   deploy --version
 
@@ -27,6 +27,8 @@ Options:
   -e             Deploys only the encrypted ZIP file, and no other file. This
                  must be used in conjunction with `-p`, otherwise the password
                  for the output file cannot be obtained.
+  --dry-run      Performs a practice run of the operation, without actually
+                 copying any file.
 
 Arguments:
   <specfile>      Path to the YAML deployment specification file to use for
@@ -87,17 +89,12 @@ Each element is a mapping with the following fields:
 
 import errno
 import os
-import re
-import uuid
 import secrets
 import string
-import subprocess
 import sys
-import yaml
 
 from collections import namedtuple
 from ftplib import FTP, error_perm
-from glob import glob
 from shutil import (
     copy,
     copytree,
@@ -109,7 +106,6 @@ from pyminizip import compress_multiple
 
 from docopt import docopt
 
-from parsimonious.exceptions import ParseError
 from parsimonious.grammar import Grammar
 from parsimonious.nodes import NodeVisitor
 
@@ -120,10 +116,10 @@ else:
 
 from azure.storage.blob import (
     BlobServiceClient,
-    BlobClient,
-    ContainerClient,
 )
 
+from .spec import get_specs_from_file, get_things, getenv
+
 NetLoc = namedtuple('NetLoc', [
     'username',  # type: str
     'password',  # type: str
@@ -169,6 +165,17 @@ class NetLocRuleVisitor(NodeVisitor):
 
 def copy_local_thing(source, destination):
     '''Copies a local file or a directory to the destination.'''
+    # Normalize file paths. Enables comparisons without risking trivial
+    # differences distorting results (e.g. "./password.txt" != "password.txt")
+    source = os.path.normpath(source)
+    destination = os.path.normpath(destination)
+    if destination == '.':
+        # Copying to local directory, nothing to do
+        return
+    if source == destination:
+        # Source and destination are the same. Nothing to do
+        return
+
     try:
         os.makedirs(destination)
     except OSError as e:
@@ -267,24 +274,6 @@ def create_password():
     return ''.join(secrets.choice(alphabet) for i in range(8))
 
 
-# Describe a (set of) files that should be deployed with an AlazarTech
-# product. See module documentation for details.
-DeploymentSpecification = namedtuple('DeploymentSpecification', [
-    'pattern',   # type: str
-    'compress',  # type: bool
-    'encrypt',   # type: bool
-    'many',      # type: bool
-    'sign',      # type: bool
-    'destdir',   # type: str
-    ])
-
-def getenv(env_var):
-    '''Queries the environment variable named `env_var`, and raises an exception if
-    it is not set.'''
-    if env_var not in os.environ:
-        raise ValueError("{} environment variable is not set".format(env_var))
-    return os.environ[env_var]
-
 def sign_file(filepath):
     '''Ditially signs a file'''
     server = getenv('ALAZARSIGN_SERVER')
@@ -298,39 +287,8 @@ def sign_file(filepath):
     with open(filepath, 'wb') as f:
         f.write(r.content)
 
-def get_things(spec):
-    '''Returns the list of files/directories to copy based on a spec'''
-    thinglist = glob(spec.pattern)
-    if not thinglist:
-        raise ValueError("Pattern {} matched no element".format(spec.pattern))
-    if not spec.many and len(thinglist) > 1:
-        raise ValueError(
-            "Pattern {} matched more than one element: {}".format(spec.pattern, thinglist)
-        )
-    return thinglist
-
-
-def get_specs_from_file(specspath):
-    output = []
-    with open(specspath, 'r') as sfile:
-        specslist = yaml.load(sfile, Loader=yaml.FullLoader)
-        for specsdict in specslist:
-            if 'many' not in specsdict:
-                specsdict['many'] = False
-            if 'compress' not in specsdict:
-                specsdict['compress'] = False
-            if 'encrypt' not in specsdict:
-                specsdict['encrypt'] = False
-            if 'sign' not in specsdict:
-                specsdict['sign'] = False
-            if 'destdir' not in specsdict:
-                specsdict['destdir'] = '/'
-            output.append(DeploymentSpecification(**specsdict))
-    return output
-
-
 def deploy(specsfile, name, destination, passfile,
-           encrypted_only):
+           encrypted_only, dry_run):
     '''Deploys a set of files according to given specifications.
 
     Parameters
@@ -348,6 +306,9 @@ def deploy(specsfile, name, destination, passfile,
     '''
     specs = get_specs_from_file(specsfile)
 
+    if dry_run:
+        print("Performing a dry run. Tasks that would have been run:")
+
     # Sign files that need it
     specs_to_sign = [spec for spec in specs if spec.sign]
     for spec in specs_to_sign:
@@ -355,8 +316,10 @@ def deploy(specsfile, name, destination, passfile,
             # Check that thing is a file, and not a directory
             if not os.path.isfile(thing):
                 raise ValueError("Thing to sign is not a file {}".format(thing))
-
-            sign_file(thing)
+            if dry_run:
+                print("- sign {}".format(thing))
+            else:
+                sign_file(thing)
 
 
     specs_to_compress = [spec for spec in specs if spec.compress]
@@ -376,25 +339,40 @@ def deploy(specsfile, name, destination, passfile,
             if passfile:
                 with open(passfile, 'r') as passf:
                     password = passf.readline().strip()
-            with open('password.txt', 'w') as passf:
-                passf.write(password)
+            if dry_run:
+                print("- save password {} to password.txt")
+            else:
+                with open('password.txt', 'w') as passf:
+                    passf.write(password)
             if not encrypted_only:
-                copy_thing('password.txt', destination)
+                if dry_run:
+                    print("- copy password.txt to {}".format(destination))
+                else:
+                    copy_thing('password.txt', destination)
 
         things_to_compress = []
         for spec in specs_to_compress:
             things = get_things(spec)
             things_to_compress.extend(things)
-        compress_multiple(things_to_compress, [], "{}.zip".format(name), password, 9)
+        if dry_run:
+            print("- compress {} into {}.zip with password {}".format(things_to_compress, name, password))
+        else:
+            compress_multiple(things_to_compress, [], "{}.zip".format(name), password, 9)
 
-        copy_thing("{}.zip".format(name), destination)
+        if dry_run:
+            print("- copy {}.zip to {}".format(name, destination))
+        else:
+            copy_thing("{}.zip".format(name), destination)
 
 
     # Copy regular things
     if not encrypted_only:
         for spec in specs:
             for thing in get_things(spec):
-                copy_thing(thing, "{}/{}".format(destination, spec.destdir))
+                if dry_run:
+                    print("- copy {} to {}/{}".format(thing, destination, spec.destdir))
+                else:
+                    copy_thing(thing, "{}/{}".format(destination, spec.destdir))
 
 
 def main():
@@ -405,7 +383,8 @@ def main():
         name=arguments['<package_name>'],
         destination=arguments['<destination>'],
         passfile=arguments['-p'],
-        encrypted_only=arguments['-e'])
+        encrypted_only=arguments['-e'],
+        dry_run=arguments['--dry-run'])
 
 
 if __name__ == "__main__":
